function [h,A,Canvas,patches,pax] = scplot(x, y, h, l1, l2, t)
%[h,A,Canvas,patches,pax] = scplot(x, y, h, l1, l2, t)
% Creates scatter plots of the columns of X.
% On the diagonal, make a histogram.
% Y provides optional class labels.
% Inputs:
%  x and y are two vectors
%  h is the figure handle
%  l1 and l2 are optional axis labels
%  t optional title
%
%   [h,A,Canvas,patches,pax] returns a matrix of handles
%   to the objects created in h, a matrix of handles to the individual
%   subaxes in A, a handle to big (invisible) axes that frame the
%   subaxes in Canvas, a matrix of handles for the histogram plots in
%   P, and a matrix of handles for invisible axes that control the
%   histogram axes scales in pax.  Canvas is left as the CurrentAxes so
%   that a subsequent TITLE, XLABEL, or YLABEL will be centered with
%   respect to the matrix of axes.

% Isabelle Guyon -- isabelle@clopinet.com -- April 2013

if nargin<3, h=figure('Position', [0 245 560 560]); end
if nargin<4, l1='A'; end
if nargin<5, l2='B'; end
if nargin<6, t=''; end

l{1}='A';
l{2}='B';

X=[x y];
Y=[];
pos = get(h, 'Position');
pos(4)=pos(3);
set(h, 'Position', pos);

if length(x)~=length(y), error('Input vectors must have the same length'); end

[p,n]=size(X);
if isempty(Y), Y=zeros(p,1); end

% Assumes 2 classes +-1; red is +1; blue is -1 or 0
if min(min(Y))==-1 
    Y=(Y+1)/2;
end
col='cbrkgmcrbkgmcrbkgmcrbkgmcrbkgm';
sym='+..o++ooooooddddddxxxxxxssssss';
        
if ndims(X)>2 | ndims(Y)>2, error('X an Y must be 2 dim'); help scatterplot; end
if (ndims(X)==2) & any(size(X)==1), X = X(:); end
if length(Y)~=p, error('Bad Y length'); help scatterplot; end
if (p==0 | n==0) 
    if nargout>0, h = []; A = []; Canvas = []; end
    return; 
end
    
coln=length(col);
cmap=make_cmap(col);

% Create Canvas and make it invisible
Canvas = newplot;
next = lower(get(Canvas,'NextPlot'));
hold_state = ishold;
set(Canvas,'Visible','off','color','none')
markersize = get(0,'defaultlinemarkersize');

% Find class labels
cl=unique(Y);
cl_num=length(cl);
cl_idx=cell(cl_num,1);
for k=0:cl_num-1
    cl_idx{k+1}=find(Y==k);
end

% Create axes and plot
A = zeros(n,n);
pos = get(Canvas,'Position');
width = pos(3)/n;
height = pos(4)/n;
space = .02; % 2 percent space between axes
pos(1:2) = pos(1:2) + space*[width height];

%%%%%%%%%[m,q,k] = size(y);
xlim = zeros([n n 2]);
ylim = zeros([n n 2]);
for i=n:-1:1,
    for j=n:-1:1,
        axPos = [pos(1)+(j-1)*width pos(2)+(n-i)*height width*(1-space) height*(1-space)];
        findax = findobj(gcf,'Type','axes','Position',axPos);
        if isempty(findax),
            A(i,j) = axes('Position',axPos);
            set(A(i,j),'visible','on');
        else
            A(i,j) = findax(1);
        end
        for k=1:cl_num
            SS=sym(mod(k, coln)+1);
            hh(i,j,:) = plot(X(cl_idx{k},j), X(cl_idx{k},i), [col(mod(k, coln)+1) SS])';
            hold on
            if SS=='.', 
                set(hh(i,j,:),'markersize',1.5*markersize);
            else
                set(hh(i,j,:),'markersize',0.5*markersize);
            end
        end

        set(A(i,j),'xlimmode','auto','ylimmode','auto','xgrid','off','ygrid','off')
        xlim(i,j,:) = get(gca,'xlim');
        ylim(i,j,:) = get(gca,'ylim');
    end
end


xlimmin = min(xlim(:,:,1),[],1); xlimmax = max(xlim(:,:,2),[],1);
ylimmin = min(ylim(:,:,1),[],2); ylimmax = max(ylim(:,:,2),[],2);


% Try to be smart about axes limits and labels.  Set all the limits of a
% row or column to be the same and inset the tick marks by 10 percent.
inset = .15;
for i=1:n,
  set(A(i,1),'ylim',[ylimmin(i,1) ylimmax(i,1)])
  dy = diff(get(A(i,1),'ylim'))*inset;
  set(A(i,:),'ylim',[ylimmin(i,1)-dy ylimmax(i,1)+dy])
end
for j=1:n,
  set(A(1,j),'xlim',[xlimmin(1,j) xlimmax(1,j)])
  dx = diff(get(A(1,j),'xlim'))*inset;
  set(A(:,j),'xlim',[xlimmin(1,j)-dx xlimmax(1,j)+dx])
end

set(A(1:n-1,:),'xticklabel','')
set(A(:,2:n),'yticklabel','')
set(Canvas,'XTick',get(A(n,1),'xtick'),'YTick',get(A(n,1),'ytick'), ...
          'userdata',A,'tag','PlotMatrixCanvas')

% Put a histogram on the diagonal 
nx=100;
for i=n:-1:1,

    histax = axes('Position',get(A(i,i),'Position'));
    nnn=[];
    delta=(xlimmax(i)-xlimmin(i))/nx;
    xx=[xlimmin(i)+delta/2:delta:xlimmax(i)];
    for k=1:cl_num
        nn = hist(X(cl_idx{k},i), xx);
        nnn = [nnn; nn];
    end
    patches(i,:) = bar(xx,nnn','hist');
    H(i)= hist_entropy(nnn);
    title(A(i,i), sprintf('Entropy(%s) = %5.2g', l{i}, H(i)), 'Fontsize', 16, 'FontWeight', 'bold');
    colormap(cmap(1:cl_num,:));
    set(histax,'xtick',[],'ytick',[],'xgrid','off','ygrid','off');
    %set(histax,'xlim',[xlimmin(1,i)-dx xlimmax(1,i)+dx])
    if (j-1)>0
        set(histax,'xlim', get(A(j-1, i), 'xlim'));
    else
        set(histax,'xlim', get(A(j+1, i), 'xlim'));
    end
    set(histax,'ylim',[0 8])
    pax(i) = histax;  % A handles for histograms
    set(A(i,i), 'xtick',[],'ytick',[],'xgrid','off','ygrid','off');
    if i==n
        camroll(90)
        axis ij
        set(A(j-1, i), 'visible', 'off');
        cla(A(j-1, i));
    end
        
end

  patches = patches';

% Make Canvas the CurrentAxes
set(gcf,'CurrentAx',Canvas)
if ~hold_state,
   set(gcf,'NextPlot','replace')
end

% Also set Title and X/YLabel visibility to on and strings to empty
set([get(Canvas,'Title'); get(Canvas,'XLabel'); get(Canvas,'YLabel')], ...
 'String','','Visible','on')

xlabel(A(2,1), l1, 'Fontsize', 16, 'FontWeight', 'bold')
ylabel(A(2,1), l2, 'Fontsize', 16, 'FontWeight', 'bold')
types={'Categorical', 'Binary', 'Numerical'};
for tt=1:length(types)
    if ~isempty(strfind(l1, types{tt})), l1=types{tt}; end
    if ~isempty(strfind(l2, types{tt})), l2=types{tt}; end
end
n=min(1000, length(x));
[r, p]=corrcoef(x(1:n), y(1:n));
t = [t sprintf('\nCORR=%5.2f (pval=%5.2f)', r(1,2), p(1,2))];
[r, p, ttl]=itest(x(1:n), y(1:n), l1, l2);
t = [t sprintf('\n%s=%5.2f (pval=%5.2f)', upper(ttl), r, p)];
%[p, r]=fasthsic(x(1:n), y(1:n));
%t = [t sprintf('\nHSIC=%5.2f (pval=%5.2f)', r, p)];
t = [t sprintf('\nDiffEntropy=%5.2f', H(1)-H(2))];
t = [t sprintf('\nIGCI=%5.2f', igci(x, y, 1, 2))];
text(.55, .95, textwrap({t},30), 'FontSize', 16, 'FontWeight', 'bold')

if nargout~=0,
  h = hh;
end

return

function cmap=make_cmap(col)

% Red 1 0 0
% Green 0 1 0
% Blue 0 0 1
% black 0 0 0
% magenta 1 0 1
% cyan 0 1 1
% Yellow 1 1 0
colcode='rgbkmc';
colmat=[1 0 0; 0 1 0; 0 0 1; 0 0 0; 1 0 1; 0 1 1];

for k=2:length(col)
    pos=strfind(colcode, col(k));
    cmap(k-1,:)=colmat(pos,:);
end
